@@ -1,6 +1,6 @@
 // L2TPNS Clustering Stuff
 
-char const *cvs_id_cluster = "$Id: cluster.c,v 1.49 2005/12/05 14:10:42 bodea Exp $";
+char const *cvs_id_cluster = "$Id: cluster.c,v 1.50.2.1 2006/12/02 14:09:14 bodea Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -1347,7 +1347,11 @@ static int cluster_process_heartbeat(uint8_t *data, int size, int more, uint8_t
 		return -1; // Ignore it??
 	}
 
-		// Ok. It's a heartbeat packet from a cluster master!
+	if (size > sizeof(past_hearts[0].data)) {
+		LOG(0, 0, 0, "Received an oversize heartbeat from %s (%d)!\n", fmtaddr(addr, 0), size);
+		return -1;
+	}
+
 	if (s < sizeof(*h))
 		goto shortpacket;
 
@@ -1655,7 +1659,11 @@ int processcluster(uint8_t *data, int size, in_addr_t addr)
 
 			STAT(recv_forward);
 			if (type == C_FORWARD_DAE)
-				processdae(p, s, &a, sizeof(a));
+			{
+				struct in_addr local;
+				local.s_addr = config->bind_address ? config->bind_address : my_address;
+				processdae(p, s, &a, sizeof(a), &local);
+			}
 			else
 				processudp(p, s, &a);
 
