@@ -1,6 +1,6 @@
 // L2TPNS: token bucket filters
 
-char const *cvs_id_tbf = "$Id: tbf.c,v 1.11 2004/12/16 08:49:53 bodea Exp $";
+char const *cvs_id_tbf = "$Id: tbf.c,v 1.13 2005/07/31 10:04:10 bodea Exp $";
 
 #include <string.h>
 #include "l2tpns.h"
@@ -159,10 +159,10 @@ void fsck_tbfs(void)
 // If we can send it right away, we do. Else we
 // try and queue it to send later. Else we drop it.
 //
-int tbf_queue_packet(int tbf_id, char * data, int size)
+int tbf_queue_packet(int tbf_id, uint8_t *data, int size)
 {
 	int i;
-	tbft * f;
+	tbft *f;
 
 	if (!filter_list)
 		return -1;
@@ -318,7 +318,7 @@ int cmd_show_tbf(struct cli_def *cli, char *command, char **argv, int argc)
 		return CLI_HELP_NO_ARGS;
 
 	if (!config->cluster_iam_master) {
-		cli_print(cli, "Can't do this on a slave.  Do it on %s",
+		cli_error(cli, "Can't do this on a slave.  Do it on %s",
 			fmtaddr(config->cluster_master_address, 0));
 
 		return CLI_OK;
